@@ -13,13 +13,21 @@
 
 from contextlib import nullcontext
 import glob
+import json
+import numpy as np
 import pandas
 import pytest
 import modin.experimental.pandas as pd
 from modin.config import Engine
 from modin.utils import get_current_execution
-from modin.pandas.test.utils import df_equals, teardown_test_files, test_data
+from modin.pandas.test.utils import (
+    df_equals,
+    get_unique_filename,
+    teardown_test_files,
+    test_data,
+)
 from modin.test.test_utils import warns_that_defaulting_to_pandas
+from modin.pandas.test.utils import parse_dates_values_by_id, time_parsing_csv_path
 
 
 @pytest.mark.skipif(
@@ -119,6 +127,29 @@ class TestCsvGlob:
             with pytest.raises(FileNotFoundError):
                 pd.read_csv_glob("s3://nyc-tlc/trip data/yellow_tripdata_2020-")
 
+    @pytest.mark.parametrize(
+        "parse_dates",
+        [pytest.param(value, id=id) for id, value in parse_dates_values_by_id.items()],
+    )
+    def test_read_single_csv_with_parse_dates(self, parse_dates):
+        try:
+            pandas_df = pandas.read_csv(time_parsing_csv_path, parse_dates=parse_dates)
+        except Exception as pandas_exception:
+            with pytest.raises(Exception) as modin_exception:
+                modin_df = pd.read_csv_glob(
+                    time_parsing_csv_path, parse_dates=parse_dates
+                )
+                # Call __repr__ on the modin df to force it to materialize.
+                repr(modin_df)
+            assert isinstance(
+                modin_exception.value, type(pandas_exception)
+            ), "Got Modin Exception type {}, but pandas Exception type {} was expected".format(
+                type(modin_exception.value), type(pandas_exception)
+            )
+        else:
+            modin_df = pd.read_csv_glob(time_parsing_csv_path, parse_dates=parse_dates)
+            df_equals(modin_df, pandas_df)
+
 
 @pytest.mark.skipif(
     Engine.get() != "Ray", reason="Currently only support Ray engine for glob paths."
@@ -199,3 +230,93 @@ def test_distributed_pickling(filename, compression):
 
     pickle_files = glob.glob(filename)
     teardown_test_files(pickle_files)
+
+
+@pytest.mark.skipif(
+    not Engine.get() == "Ray",
+    reason=f"{Engine.get()} does not have experimental read_custom_text API",
+)
+def test_read_custom_json_text():
+    filename = get_unique_filename(extension="json")
+
+    def _generate_json(file_name, nrows, ncols):
+        data = np.random.rand(nrows, ncols)
+        df = pandas.DataFrame(data, columns=[f"col{x}" for x in range(ncols)])
+        df.to_json(file_name, lines=True, orient="records")
+
+    _generate_json(filename, 64, 8)
+
+    # Custom parser allows us to add some specifics to reading files,
+    # which is not available through the ready-made API.
+    # For example, the parser allows us to reduce the amount of RAM
+    # required for reading by selecting a subset of columns.
+    def _custom_parser(io_input, **kwargs):
+        result = {"col0": [], "col1": [], "col3": []}
+        for line in io_input:
+            # for example, simjson can be used here
+            obj = json.loads(line)
+            for key in result:
+                result[key].append(obj[key])
+        return pandas.DataFrame(result).rename(columns={"col0": "testID"})
+
+    df1 = pd.read_custom_text(
+        filename,
+        columns=["testID", "col1", "col3"],
+        custom_parser=_custom_parser,
+        is_quoting=False,
+    )
+    df2 = pd.read_json(filename, lines=True)[["col0", "col1", "col3"]].rename(
+        columns={"col0": "testID"}
+    )
+    df_equals(df1, df2)
+
+
+@pytest.mark.skipif(
+    not Engine.get() == "Ray",
+    reason=f"{Engine.get()} does not have experimental API",
+)
+def test_read_evaluated_dict():
+    filename = get_unique_filename(extension="json")
+
+    def _generate_evaluated_dict(file_name, nrows, ncols):
+        result = {}
+        keys = [f"col{x}" for x in range(ncols)]
+
+        with open(file_name, mode="w") as _file:
+            for i in range(nrows):
+                data = np.random.rand(ncols)
+                for idx, key in enumerate(keys):
+                    result[key] = data[idx]
+                _file.write(str(result))
+                _file.write("\n")
+
+    _generate_evaluated_dict(filename, 64, 8)
+
+    # This parser allows us to read a format not supported by other reading functions
+    def _custom_parser(io_input, **kwargs):
+        cat_list = []
+        asin_list = []
+        for line in io_input:
+            obj = eval(line)
+            cat_list.append(obj["col1"])
+            asin_list.append(obj["col2"])
+        return pandas.DataFrame({"col1": asin_list, "col2": cat_list})
+
+    df1 = pd.read_custom_text(
+        filename,
+        columns=["col1", "col2"],
+        custom_parser=_custom_parser,
+    )
+    assert df1.shape == (64, 2)
+
+    def columns_callback(io_input, **kwargs):
+        columns = None
+        for line in io_input:
+            columns = list(eval(line).keys())[1:3]
+            break
+        return columns
+
+    df2 = pd.read_custom_text(
+        filename, columns=columns_callback, custom_parser=_custom_parser
+    )
+    df_equals(df1, df2)
