@@ -20,7 +20,7 @@ from modin.distributed.dataframe.pandas import unwrap_partitions, from_partition
 from modin.config import Engine, NPartitions
 from modin.pandas.test.utils import df_equals
 from modin.pandas.indexing import compute_sliced_len
-from modin.data_management.factories.dispatcher import FactoryDispatcher
+from modin.core.execution.dispatching.factories.dispatcher import FactoryDispatcher
 
 PartitionClass = (
     FactoryDispatcher.get_factory().io_cls.frame_cls._partition_mgr_cls._partition_class
@@ -33,11 +33,11 @@ if Engine.get() == "Ray":
     get_func = ray.get
     FutureType = ray.ObjectRef
 elif Engine.get() == "Dask":
-    from distributed.client import default_client
+    from modin.core.execution.dask.common.engine_wrapper import DaskWrapper
     from distributed import Future
 
-    put_func = lambda x: default_client().scatter(x)  # noqa: E731
-    get_func = lambda x: x.result()  # noqa: E731
+    put_func = lambda x: DaskWrapper.put(x)  # noqa: E731
+    get_func = lambda x: DaskWrapper.materialize(x)  # noqa: E731
     FutureType = Future
 elif Engine.get() == "Python":
     put_func = lambda x: x  # noqa: E731
@@ -55,7 +55,7 @@ pd.DataFrame([])
 
 @pytest.mark.parametrize("axis", [None, 0, 1])
 def test_unwrap_partitions(axis):
-    data = np.random.randint(0, 100, size=(2 ** 16, 2 ** 8))
+    data = np.random.randint(0, 100, size=(2**16, 2**8))
     df = pd.DataFrame(data)
 
     if axis is None:
@@ -90,15 +90,10 @@ def test_unwrap_partitions(axis):
         actual_axis_partitions = unwrap_partitions(df, axis=axis)
         assert len(expected_axis_partitions) == len(actual_axis_partitions)
         for item_idx in range(len(expected_axis_partitions)):
-            if Engine.get() == "Ray":
+            if Engine.get() in ["Ray", "Dask"]:
                 df_equals(
-                    ray.get(expected_axis_partitions[item_idx]),
-                    ray.get(actual_axis_partitions[item_idx]),
-                )
-            if Engine.get() == "Dask":
-                df_equals(
-                    expected_axis_partitions[item_idx].result(),
-                    actual_axis_partitions[item_idx].result(),
+                    get_func(expected_axis_partitions[item_idx]),
+                    get_func(actual_axis_partitions[item_idx]),
                 )
 
 
@@ -108,8 +103,8 @@ def test_unwrap_partitions(axis):
 @pytest.mark.parametrize("index", [None, "index"])
 @pytest.mark.parametrize("axis", [None, 0, 1])
 def test_from_partitions(axis, index, columns, row_lengths, column_widths):
-    num_rows = 2 ** 16
-    num_cols = 2 ** 8
+    num_rows = 2**16
+    num_cols = 2**8
     data = np.random.randint(0, 100, size=(num_rows, num_cols))
     df1, df2 = pandas.DataFrame(data), pandas.DataFrame(data)
     expected_df = pandas.concat([df1, df2], axis=1 if axis is None else axis)
@@ -133,15 +128,14 @@ def test_from_partitions(axis, index, columns, row_lengths, column_widths):
 
     if Engine.get() == "Ray":
         if axis is None:
-            futures = [[ray.put(df1), ray.put(df2)]]
+            futures = [[put_func(df1), put_func(df2)]]
         else:
-            futures = [ray.put(df1), ray.put(df2)]
+            futures = [put_func(df1), put_func(df2)]
     if Engine.get() == "Dask":
-        client = default_client()
         if axis is None:
-            futures = [client.scatter([df1, df2], hash=False)]
+            futures = [put_func([df1, df2], hash=False)]
         else:
-            futures = client.scatter([df1, df2], hash=False)
+            futures = put_func([df1, df2], hash=False)
     actual_df = from_partitions(
         futures,
         axis,
@@ -157,8 +151,8 @@ def test_from_partitions(axis, index, columns, row_lengths, column_widths):
 @pytest.mark.parametrize("index", ["original_idx", "new_idx"])
 @pytest.mark.parametrize("axis", [None, 0, 1])
 def test_from_partitions_mismatched_labels(axis, index, columns):
-    num_rows = 2 ** 16
-    num_cols = 2 ** 8
+    num_rows = 2**16
+    num_cols = 2**8
     expected_df = pd.DataFrame(np.random.randint(0, 100, size=(num_rows, num_cols)))
     partitions = unwrap_partitions(expected_df, axis=axis)
 
@@ -179,13 +173,11 @@ def test_from_partitions_mismatched_labels(axis, index, columns):
     df_equals(expected_df, actual_df)
 
 
-@pytest.mark.parametrize("row_indices", [[0, 2], slice(None)])
-@pytest.mark.parametrize("col_indices", [[0, 2], slice(None)])
+@pytest.mark.parametrize("row_labels", [[0, 2], slice(None)])
+@pytest.mark.parametrize("col_labels", [[0, 2], slice(None)])
 @pytest.mark.parametrize("is_length_future", [False, True])
 @pytest.mark.parametrize("is_width_future", [False, True])
-def test_mask_preserve_cache(
-    row_indices, col_indices, is_length_future, is_width_future
-):
+def test_mask_preserve_cache(row_labels, col_labels, is_length_future, is_width_future):
     def deserialize(obj):
         if isinstance(obj, FutureType):
             return get_func(obj)
@@ -206,11 +198,11 @@ def test_mask_preserve_cache(
 
     source_partition = PartitionClass(obj_id, *partition_shape)
     masked_partition = source_partition.mask(
-        row_indices=row_indices, col_indices=col_indices
+        row_labels=row_labels, col_labels=col_labels
     )
 
-    expected_length = compute_length(row_indices, len(df))
-    expected_width = compute_length(col_indices, len(df.columns))
+    expected_length = compute_length(row_labels, len(df))
+    expected_width = compute_length(col_labels, len(df.columns))
 
     # Check that the cache is preserved
     assert expected_length == deserialize(masked_partition._length_cache)
